// BenLincoln.DaVinci.GUI - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.GUI.

// BenLincoln.DaVinci.GUI is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci.GUI is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci.GUI (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace BenLincoln.DaVinci.GUI
{
    public class ClickableThumbnailContextMenu : System.Windows.Forms.ContextMenu
    {
        protected System.EventHandler _PopupEventHandler;
        protected System.EventHandler _CollapseEventHandler;

        public ClickableThumbnailContextMenu()
            : base()
        {
        }

        ~ClickableThumbnailContextMenu()
        {
            this.Dispose(true);
        }

        protected override void Dispose(bool disposing)
        {
            if (_PopupEventHandler != null)
            {
                this.Popup -= _PopupEventHandler;
                _PopupEventHandler = null;
            }
            if (_CollapseEventHandler != null)
            {
                this.Collapse -= _CollapseEventHandler;
                _CollapseEventHandler = null;
            }

            if (this.MenuItems.Count > 0)
            {
                for (int i = 0; i < this.MenuItems.Count; i++)
                {
                    if (this.MenuItems[i] != null)
                    {
                        try
                        {
                            this.MenuItems[i].Dispose();
                            //this.MenuItems[i] = null;
                            this.MenuItems.Remove(this.MenuItems[i]);
                        }
                        catch (Exception ex)
                        {
                            // do nothing
                        }
                    }
                }
            }

            base.Dispose(disposing);
        }

        public void SetPopupEventHandler(System.EventHandler handler)
        {
            if (_PopupEventHandler != null)
            {
                this.Popup -= _PopupEventHandler;
            }
            this.Popup += handler;
            _PopupEventHandler = handler;
        }

        public void SetCollapseEventHandler(System.EventHandler handler)
        {
            if (_CollapseEventHandler != null)
            {
                this.Collapse -= _CollapseEventHandler;
            }
            this.Collapse += handler;
            _CollapseEventHandler = handler;
        }

    }
}
