// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public enum CalculatedChannelCombinationMethod
    {
        Ratio,
        Difference,
        NormalizedDifference
        //NormalizedDifferencePositive,
        //NormalizedDifferenceNegative,
        //NormalizedDifferenceCenteredOnZeroPointFive
    }

    public struct CalculatedChannelSet
    {
        public CalculatedChannelCombinationConfig Config;
        public GreyscaleChannel[] Channels;
    }

    public class CalculatedChannelCombinationConfig
    {
        protected string _CombinationConfigName;
        //protected bool _Generate;

        protected CalculatedChannelCombinationMethod _CombineMethod;

        protected TransformationProfileCollection _InputTransformations;
        protected TransformationProfileCollection _OutputTransformations;

        protected int _MaxRecursionDepth;
        protected bool _GenerateReciprocal;
        //protected bool _AbsoluteValueOutput;
        //protected NormalizationMode _OutputNormalization;
        //protected NormalizationMode _NAAVNormalization;
        protected bool _UseAsLuminanceChannel;
        protected bool _UseAsStandardThreeChannelChannel;

        protected bool _GenerateThreeChannelSimilarMotion;
        protected bool _GenerateThreeChannelObliqueMotion;
        protected bool _GenerateThreeChannelOtherPermutations;
        protected bool _GenerateThreeChannelReciprocal;
        protected bool _UseThreeChannelPermuatationsInLumaColourImages;

        #region Properties

        public string CombinationConfigName
        {
            get
            {
                return _CombinationConfigName;
            }
            set
            {
                _CombinationConfigName = value;
            }
        }

        public CalculatedChannelCombinationMethod CombineMethod
        {
            get
            {
                return _CombineMethod;
            }
            set
            {
                _CombineMethod = value;
            }
        }

        public TransformationProfileCollection InputTransformations
        {
            get
            {
                return _InputTransformations;
            }
            set
            {
                _InputTransformations = value;
            }
        }

        public TransformationProfileCollection OutputTransformations
        {
            get
            {
                return _OutputTransformations;
            }
            set
            {
                _OutputTransformations = value;
            }
        }

        //public string CombinationConfigName
        //{
        //    get
        //    {
        //        return _CombinationConfigName;
        //    }
        //    set
        //    {
        //        _CombinationConfigName = value;
        //    }
        //}

        public int MaxRecursionDepth
        {
            get
            {
                return _MaxRecursionDepth;
            }
            set
            {
                _MaxRecursionDepth = value;
            }
        }

        //public bool Generate
        //{
        //    get
        //    {
        //        return _Generate;
        //    }
        //    set
        //    {
        //        _Generate = value;
        //    }
        //}

        public bool GenerateReciprocal
        {
            get
            {
                return _GenerateReciprocal;
            }
            set
            {
                _GenerateReciprocal = value;
            }
        }

        //public bool AbsoluteValueOutput
        //{
        //    get
        //    {
        //        return _AbsoluteValueOutput;
        //    }
        //    set
        //    {
        //        _AbsoluteValueOutput = value;
        //    }
        //}

        //public bool OutputAbsoluteValue
        //{
        //    get
        //    {
        //        return _OutputAbsoluteValue;
        //    }
        //    set
        //    {
        //        _OutputAbsoluteValue = value;
        //    }
        //}

        //public NormalizationMode OutputNormalization
        //{
        //    get
        //    {
        //        return _OutputNormalization;
        //    }
        //    set
        //    {
        //        _OutputNormalization = value;
        //    }
        //}

        //public NormalizationMode NAAVNormalization
        //{
        //    get
        //    {
        //        return _NAAVNormalization;
        //    }
        //    set
        //    {
        //        _NAAVNormalization = value;
        //    }
        //}

        public bool UseAsLuminanceChannel
        {
            get
            {
                return _UseAsLuminanceChannel;
            }
            set
            {
                _UseAsLuminanceChannel = value;
            }
        }

        public bool UseAsStandardThreeChannelChannel
        {
            get
            {
                return _UseAsStandardThreeChannelChannel;
            }
            set
            {
                _UseAsStandardThreeChannelChannel = value;
            }
        }

        /// <summary>
        /// Controls whether or not three-channel permutations will be created in which the source spectral
        /// bands used in each channel increase in frequency between each channel (for example, NIR:R - R:G - G:B, or 
        /// NIR:G - R:B - G:UVA).
        /// </summary>
        public bool GenerateThreeChannelSimilarMotion
        {
            get
            {
                return _GenerateThreeChannelSimilarMotion;
            }
            set
            {
                _GenerateThreeChannelSimilarMotion = value;
            }
        }

        /// <summary>
        /// Controls whether or not three-channel permutations will be created in which the first source 
        /// spectral band remains the same between channels, but the second changes (for example, NIR:R - NIR:G - NIR:B)
        /// </summary>
        public bool GenerateThreeChannelObliqueMotion
        {
            get
            {
                return _GenerateThreeChannelObliqueMotion;
            }
            set
            {
                _GenerateThreeChannelObliqueMotion = value;
            }
        }

        /// <summary>
        /// Controls whether or not three-channel permutations will be created based on all ascending combinations
        /// of channels which do not satisfy the criteria for being either "similar motion" or "oblique motion". 
        /// For example, NIR:R - NIR:G - G:UVA.
        /// </summary>
        public bool GenerateThreeChannelOtherPermutations
        {
            get
            {
                return _GenerateThreeChannelOtherPermutations;
            }
            set
            {
                _GenerateThreeChannelOtherPermutations = value;
            }
        }

        /// <summary>
        /// Controls whether or not three-channel permutations will be created using the reciprocal variations of 
        /// the calculated channels. For example, if Generate3CObliqueMotion is set to true as well, then in 
        /// addition to generating NIR:R - NIR:G - NIR:B, the variation R:NIR - G:NIR - B:NIR would also be 
        /// generated.
        /// </summary>
        public bool GenerateThreeChannelReciprocal
        {
            get
            {
                return _GenerateThreeChannelReciprocal;
            }
            set
            {
                _GenerateThreeChannelReciprocal = value;
            }
        }

        /// <summary>
        /// Controls whether or not the three-channel permutations generated will be used in 
        /// luma/colour images.
        /// </summary>
        public bool UseThreeChannelPermuatationsInLumaColourImages
        {
            get
            {
                return _UseThreeChannelPermuatationsInLumaColourImages;
            }
            set
            {
                _UseThreeChannelPermuatationsInLumaColourImages = value;
            }
        }

        #endregion

        public CalculatedChannelCombinationConfig()
        {
            CombinationConfigName = "";
            //_Generate = false;
            _MaxRecursionDepth = 1;
            _GenerateReciprocal = false;
            InputTransformations = new TransformationProfileCollection();
            OutputTransformations = new TransformationProfileCollection();
            //_AbsoluteValueOutput = false;
            //_OutputAbsoluteValue = true;
            //_OutputNormalization = NormalizationMode.None;
            //NAAVNormalization = NormalizationMode.Standard;
            _UseAsLuminanceChannel = false;
            _UseAsStandardThreeChannelChannel = false;

            GenerateThreeChannelSimilarMotion = false;
            GenerateThreeChannelObliqueMotion = false;
            GenerateThreeChannelOtherPermutations = false;
            GenerateThreeChannelReciprocal = false;
            UseThreeChannelPermuatationsInLumaColourImages = true;
        }

        public string GetCalculatedPermutationScriptCode(string channel1Name, string channel2Name)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append("= ");
            //if (AbsoluteValueOutput)
            //{
            //    builder.Append("abs(");
            //}
            switch (CombineMethod)
            {
                case CalculatedChannelCombinationMethod.Ratio:
                    builder.Append("%ChannelByName:CHANNEL1% / %ChannelByName:CHANNEL2%");
                    break;
                case CalculatedChannelCombinationMethod.Difference:
                    builder.Append("%ChannelByName:CHANNEL1% - %ChannelByName:CHANNEL2%");
                    break;
                case CalculatedChannelCombinationMethod.NormalizedDifference:
                    builder.Append("ndi(%ChannelByName:CHANNEL1%, %ChannelByName:CHANNEL2%)");
                    break;
                //case CalculatedChannelCombinationMethod.NormalizedDifferencePositive:
                //    builder.Append("clipdata(ndi(%ChannelByName:CHANNEL1%, %ChannelByName:CHANNEL2%), min = 0.0, max = 1.0)");
                //    break;
                //case CalculatedChannelCombinationMethod.NormalizedDifferenceNegative:
                //    builder.Append("abs(clipdata(ndi(%ChannelByName:CHANNEL1%, %ChannelByName:CHANNEL2%), min = -1.0, max = 0.0))");
                //    break;
                //case CalculatedChannelCombinationMethod.NormalizedDifferenceCenteredOnZeroPointFive:
                //    if ((NAAVNormalization == NormalizationMode.None) || (NAAVNormalization == NormalizationMode.Standard))
                //    {
                //        builder.Append("naav(ndi(%ChannelByName:CHANNEL1%, %ChannelByName:CHANNEL2%), ubound = 0.5, lbound = -0.5, independent = 1) + 0.5");
                //    }
                //    else
                //    {
                //        builder.Append("naav((");
                //        builder.Append(OutputImage.WrapScriptObjectInNormalizationCode("clipdata(ndi(%ChannelByName:CHANNEL1%, %ChannelByName:CHANNEL2%), min = 0.0, max = 1.0)", NAAVNormalization));
                //        builder.Append(" + ");
                //        builder.Append(OutputImage.WrapScriptObjectInNormalizationCode("((abs(clipdata(ndi(%ChannelByName:CHANNEL1%, %ChannelByName:CHANNEL2%), min = -1.0, max = 0.0))) * -1.0)", NAAVNormalization));
                //        builder.Append("), ubound = 0.5, lbound = -0.5, independent = 1) + 0.5");
                //    }
                //    break;
                default:
                    throw new ArgumentException("The band combination method '" + CombineMethod.ToString() + "' is not supported by this function.");
                    break;
            }
            //if (AbsoluteValueOutput)
            //{
            //    builder.Append(")");
            //}
            string scriptCode = builder.ToString();
            scriptCode = scriptCode.Replace(Environment.NewLine, "");
            scriptCode = scriptCode.Replace("CHANNEL1", InputTransformations.WrapScriptObjectInTransformationCode(channel1Name));
            scriptCode = scriptCode.Replace("CHANNEL2", InputTransformations.WrapScriptObjectInTransformationCode(channel2Name));
            //scriptCode = "= " + OutputTransformations.WrapScriptObjectInTransformationCode(scriptCode);
            return scriptCode;
        }

        public string GetCalculatedPermutationName(string channel1Name, string channel2Name)
        {
            StringBuilder builder = new StringBuilder();
            //switch (CombineMethod)
            //{
            //    case CalculatedChannelCombinationMethod.Ratio:
            //        builder.Append("Ratio-");
            //        break;
            //    case CalculatedChannelCombinationMethod.Difference:
            //        builder.Append("Difference-");
            //        break;
            //    case CalculatedChannelCombinationMethod.NormalizedDifferencePositive:
            //        builder.Append("ND_(Pos)-");
            //        break;
            //    case CalculatedChannelCombinationMethod.NormalizedDifferenceNegative:
            //        builder.Append("ND_(Neg)-");
            //        break;
            //    case CalculatedChannelCombinationMethod.NormalizedDifferenceCenteredOnZeroPointFive:
            //        builder.Append("ND-");
            //        break;
            //    default:
            //        throw new ArgumentException("The band combination method '" + CombineMethod.ToString() + "' is not supported by this function.");
            //        break;
            //}
            builder.Append(CombinationConfigName);
            builder.Append("-");
            builder.Append(OutputImage.WrapNameComponentIfComplex(channel1Name));
            switch (CombineMethod)
            {
                case CalculatedChannelCombinationMethod.Ratio:
                    builder.Append("_To_");
                    break;
                case CalculatedChannelCombinationMethod.Difference:
                    builder.Append("-");
                    break;
                case CalculatedChannelCombinationMethod.NormalizedDifference:
                    builder.Append("-");
                    break;
            }
            builder.Append(OutputImage.WrapNameComponentIfComplex(channel2Name));
            return builder.ToString();
        }

        public static CalculatedChannelCombinationMethod CalculatedChannelCombinationMethodFromString(string methodName)
        {
            string mName = methodName.Trim().ToLower();
            switch (mName)
            {
                case "ratio":
                    return CalculatedChannelCombinationMethod.Ratio;
                    break;
                case "difference":
                    return CalculatedChannelCombinationMethod.Difference;
                    break;
                case "normalizeddifference":
                    return CalculatedChannelCombinationMethod.NormalizedDifference;
                    break;
            }

            throw new ArgumentException("Unable to convert the string '" + methodName + "' to a recognized CalculatedChannelCombinationMethod.");
        }

        public static string GetCalculatedChannelCombinationMethodName(CalculatedChannelCombinationMethod method)
        {
            switch (method)
            {
                case CalculatedChannelCombinationMethod.Ratio:
                    return "ratio";
                    break;
                case CalculatedChannelCombinationMethod.Difference:
                    return "difference";
                    break;
                case CalculatedChannelCombinationMethod.NormalizedDifference:
                    return "normalized difference";
                    break;
            }
            return method.ToString();
        }
    }
}
