// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class ColourGradient
    {
        protected string _GradientName;
        protected GradientColour[] _Colours;

        #region Properties

        public string GradientName
        {
            get
            {
                return _GradientName;
            }
            set
            {
                _GradientName = value;
            }
        }

        public GradientColour[] Colours
        {
            get
            {
                return _Colours;
            }
            set
            {
                _Colours = value;
            }
        }

        #endregion

        public ColourGradient()
        {
            _GradientName = "";
            _Colours = new GradientColour[0];
        }

        public void AddColour(GradientColour newColour)
        {
            if ((Colours == null) || (Colours.Length == 0))
            {
                Colours = new GradientColour[1];
            }
            else
            {
                Array.Resize(ref _Colours, _Colours.Length + 1);
            }
            Colours[Colours.Length - 1] = newColour;
        }

        public string GetCreationScriptCode(string scriptTargetObjectName)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append("# Create the '");
            builder.Append(_GradientName);
            builder.Append("' colour gradient");
            builder.Append(Environment.NewLine);

            string colourFormatString = "gradient_colour_{0:00}";

            // make sure the colours are in the correct order whether the user specified them that way or not
            Array.Sort(_Colours);

            // create each colour individually
            for (int i = 0; i < Colours.Length; i++)
            {
                string colourName = String.Format(colourFormatString, i);
                builder.Append(colourName);
                builder.Append(" = clone(0.0, 1, 1, 3)");
                builder.Append(Environment.NewLine);

                // red
                builder.Append(colourName);
                builder.Append("[1, 1, 1] = ");
                builder.Append(Colours[i].Red.ToString());
                builder.Append(Environment.NewLine);

                // green
                builder.Append(colourName);
                builder.Append("[1, 1, 2] = ");
                builder.Append(Colours[i].Green.ToString());
                builder.Append(Environment.NewLine);

                // blue
                builder.Append(colourName);
                builder.Append("[1, 1, 3] = ");
                builder.Append(Colours[i].Blue.ToString());
                builder.Append(Environment.NewLine);
            }

            // create the gradient object using those colours
            builder.Append(scriptTargetObjectName);
            builder.Append(" = buildgradient(");
            for (int i = 0; i < Colours.Length; i++)
            {
                string colourName = String.Format(colourFormatString, i);
                builder.Append(Colours[i].Position);
                builder.Append(", ");
                builder.Append(colourName);
                if (i < (Colours.Length - 1))
                {
                    builder.Append(", ");
                }
            }
            builder.Append(")");
            builder.Append(Environment.NewLine);

            // for debugging only
            //builder.Append(Environment.NewLine);
            //builder.Append("# Debug output");
            //builder.Append(Environment.NewLine);
            //builder.Append("write(clone(toshort(");
            //builder.Append(scriptTargetObjectName);
            //builder.Append("), 1, 100, 1), \"Debug-Gradient-");
            //builder.Append(GradientName);
            //builder.Append(".TIF\", type=tiff, force = 1)");
            //builder.Append(Environment.NewLine);

            // clean up temporary objects
            for (int i = 0; i < Colours.Length; i++)
            {
                string colourName = String.Format(colourFormatString, i);
                builder.Append(colourName);
                builder.Append(" = 0");
                builder.Append(Environment.NewLine);
            }
            //builder.Append(Environment.NewLine);

            return builder.ToString();
        }
    }
}
