// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class CombinedBandConfig
    {
        protected BandCombineMethod _BandCombineMode;
        //protected NormalizationMode _OutputNormalization;
        protected bool _UseAsLuminanceChannel;
        protected TransformationProfileCollection _OutputTransformations;

        #region Properties

        public BandCombineMethod BandCombineMode
        {
            get
            {
                return _BandCombineMode;
            }
            set
            {
                _BandCombineMode = value;
            }
        }

        //public NormalizationMode OutputNormalization
        //{
        //    get
        //    {
        //        return _OutputNormalization;
        //    }
        //    set
        //    {
        //        _OutputNormalization = value;
        //    }
        //}

        public TransformationProfileCollection OutputTransformations
        {
            get
            {
                return _OutputTransformations;
            }
            set
            {
                _OutputTransformations = value;
            }
        }

        public bool UseAsLuminanceChannel
        {
            get
            {
                return _UseAsLuminanceChannel;
            }
            set
            {
                _UseAsLuminanceChannel = value;
            }
        }

        #endregion

        public CombinedBandConfig()
        {
            _BandCombineMode = BandCombineMethod.Average;
            //_OutputNormalization = NormalizationMode.None;
            _UseAsLuminanceChannel = false;
            OutputTransformations = new TransformationProfileCollection();
        }

        public CombinedBandConfig(BandCombineMethod bandCombineMode, TransformationProfileCollection outputTransformations)
            : this()
        {
            _BandCombineMode = bandCombineMode;
            //_OutputNormalization = outputNormalization;
            OutputTransformations = outputTransformations;
        }

        public static bool ArrayContainsBandCombineMethod(CombinedBandConfig[] configArray, BandCombineMethod method)
        {
            if ((configArray == null) || (configArray.Length == 0))
            {
                return false;
            }
            foreach (CombinedBandConfig config in configArray)
            {
                if (config.BandCombineMode == method)
                {
                    return true;
                }
            }
            return false;
        }
    }
}
