// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;

using BenLincoln.DaVinci.TransformationProfiles;

namespace BenLincoln.DaVinci
{
    public class TransformationProfileConfigurationFile : TMSBConfigurationFile
    {
        protected TransformationProfileCollection _TransformationProfiles;

        #region Properties

        public TransformationProfileCollection TransformationProfiles
        {
            get
            {
                return _TransformationProfiles;
            }
            set
            {
                _TransformationProfiles = value;
            }
        }

        #endregion

        protected enum ConfigProcessingState
        {
            Normal,
            AbsoluteValueProfile,
            AdditionProfile,
            ClippingProfile,
            DataRepositionAndScaleProfile,
            ExponentProfile,
            MultiplicationProfile,
            NormalizationAboutArbitraryValueProfile,
            NormalizationProfile,
            TransformationProfileCollection
        }

        public TransformationProfileConfigurationFile(string configFileName)
        {
            TransformationProfiles = new TransformationProfileCollection();
            TransformationProfiles.Name = "Global Transformation Profile List";
            TransformationProfiles.RequireUniqueNames = true;
            ReadData(configFileName);
        }

        protected override void ReadTypeSpecificData(Stream sourceStream)
        {
            XmlTextReader xReader = new XmlTextReader(sourceStream);
            //xReader.Settings.Schemas.Add(_FileSchema);

            string currentElementName = "";
            ConfigProcessingState currentState = ConfigProcessingState.Normal;

            AbsoluteValueProfile absoluteValueProfile = new AbsoluteValueProfile();
            AdditionProfile additionProfile = new AdditionProfile();
            ClippingProfile clippingProfile = new ClippingProfile();
            DataRepositionAndScaleProfile drasProfile = new DataRepositionAndScaleProfile();
            ExponentProfile expProfile = new ExponentProfile();
            MultiplicationProfile multiplicationProfile = new MultiplicationProfile();
            NormalizationAboutArbitraryValueProfile naavProfile = new NormalizationAboutArbitraryValueProfile();
            NormalizationProfile normalizationProfile = new NormalizationProfile();
            TransformationProfileCollection profileCollection = new TransformationProfileCollection();

            while (xReader.Read())
            {
                switch (xReader.NodeType)
                {
                    case XmlNodeType.Element:
                        currentElementName = xReader.Name;
                        switch (currentState)
                        {
                            case ConfigProcessingState.Normal:
                                switch (currentElementName)
                                {
                                    case "AbsoluteValueProfile":
                                        absoluteValueProfile = new AbsoluteValueProfile();
                                        currentState = ConfigProcessingState.AbsoluteValueProfile;
                                        break;
                                    case "AdditionProfile":
                                        additionProfile = new AdditionProfile();
                                        currentState = ConfigProcessingState.AdditionProfile;
                                        break;
                                    case "ClippingProfile":
                                        clippingProfile = new ClippingProfile();
                                        currentState = ConfigProcessingState.ClippingProfile;
                                        break;
                                    case "DataRepositionAndScaleProfile":
                                        drasProfile = new DataRepositionAndScaleProfile();
                                        currentState = ConfigProcessingState.DataRepositionAndScaleProfile;
                                        break;
                                    case "ExponentProfile":
                                        expProfile = new ExponentProfile();
                                        currentState = ConfigProcessingState.ExponentProfile;
                                        break;
                                    case "MultiplicationProfile":
                                        multiplicationProfile = new MultiplicationProfile();
                                        currentState = ConfigProcessingState.MultiplicationProfile;
                                        break;
                                    case "NormalizationAboutArbitraryValueProfile":
                                        naavProfile = new NormalizationAboutArbitraryValueProfile();
                                        currentState = ConfigProcessingState.NormalizationAboutArbitraryValueProfile;
                                        break;
                                    case "NormalizationProfile":
                                        normalizationProfile = new NormalizationProfile();
                                        currentState = ConfigProcessingState.NormalizationProfile;
                                        break;
                                    case "TransformationProfileCollection":
                                        profileCollection = new TransformationProfileCollection();
                                        currentState = ConfigProcessingState.TransformationProfileCollection;
                                        break;
                                }
                                break;
                        }
                        break;
                    case XmlNodeType.EndElement:
                        currentElementName = xReader.Name;
                        switch (currentState)
                        {
                            case ConfigProcessingState.AbsoluteValueProfile:
                                if (currentElementName == "AbsoluteValueProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(absoluteValueProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.AdditionProfile:
                                if (currentElementName == "AdditionProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(additionProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.ClippingProfile:
                                if (currentElementName == "ClippingProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(clippingProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.DataRepositionAndScaleProfile:
                                if (currentElementName == "DataRepositionAndScaleProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(drasProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.ExponentProfile:
                                if (currentElementName == "ExponentProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(expProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.MultiplicationProfile:
                                if (currentElementName == "MultiplicationProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(multiplicationProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.NormalizationAboutArbitraryValueProfile:
                                if (currentElementName == "NormalizationAboutArbitraryValueProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(naavProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.NormalizationProfile:
                                if (currentElementName == "NormalizationProfile")
                                {
                                    TransformationProfiles.AddTransformationProfile(normalizationProfile);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.TransformationProfileCollection:
                                if (currentElementName == "TransformationProfileCollection")
                                {
                                    TransformationProfiles.AddTransformationProfile(profileCollection);
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                        }
                        break;
                    case XmlNodeType.Text:
                        string currentText = xReader.Value.Trim();
                        switch (currentState)
                        {
                            case ConfigProcessingState.AbsoluteValueProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        absoluteValueProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        absoluteValueProfile.Description = currentText.Trim();
                                        break;
                                }
                                break;
                            case ConfigProcessingState.AdditionProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        additionProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        additionProfile.Description = currentText.Trim();
                                        break;
                                    case "Amount":
                                        additionProfile.Amount = float.Parse(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.ClippingProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        clippingProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        clippingProfile.Description = currentText.Trim();
                                        break;
                                    case "LowerBound":
                                        clippingProfile.ClipInputDataLower = true;
                                        clippingProfile.ClipInputDataLowerValue = float.Parse(currentText);
                                        break;
                                    case "UpperBound":
                                        clippingProfile.ClipInputDataUpper = true;
                                        clippingProfile.ClipInputDataUpperValue = float.Parse(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.DataRepositionAndScaleProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        drasProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        drasProfile.Description = currentText.Trim();
                                        break;
                                    case "OutputLowerBound":
                                        drasProfile.SetOutputLowerBound = true;
                                        drasProfile.OutputLowerBound = float.Parse(currentText);
                                        break;
                                    case "OutputUpperBound":
                                        drasProfile.SetOutputUpperBound = true;
                                        drasProfile.OutputUpperBound = float.Parse(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.ExponentProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        expProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        expProfile.Description = currentText.Trim();
                                        break;
                                    case "Amount":
                                        expProfile.Amount = float.Parse(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.MultiplicationProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        multiplicationProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        multiplicationProfile.Description = currentText.Trim();
                                        break;
                                    case "Amount":
                                        multiplicationProfile.Amount = float.Parse(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.NormalizationAboutArbitraryValueProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        naavProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        naavProfile.Description = currentText.Trim();
                                        break;
                                    case "InputCenterPoint":
                                        naavProfile.InputCenterPoint = float.Parse(currentText);
                                        break;
                                    case "NormalizeUpperAndLowerIndependently":
                                        naavProfile.NormalizeUpperAndLowerIndependently = StringToBool(currentText);
                                        break;
                                    case "OutputLowerBound":
                                        naavProfile.OutputLowerBound = float.Parse(currentText);
                                        break;
                                    case "OutputUpperBound":
                                        naavProfile.OutputUpperBound = float.Parse(currentText);
                                        break;
                                    case "Stance":
                                        naavProfile.Stance = NormalizationProfile.NormalizationStanceFromString(currentText);
                                        break;
                                    case "QuantizationStepCount":
                                        naavProfile.QuantizationStepCount = int.Parse(currentText);
                                        break;
                                    case "Threshold":
                                        naavProfile.Threshold = float.Parse(currentText);
                                        break;
                                    case "ThresholdRepetitions":
                                        naavProfile.ThresholdRepetitions = int.Parse(currentText);
                                        break;
                                    case "SearchWidth":
                                        naavProfile.SearchWidth = int.Parse(currentText);
                                        break;
                                    case "Backlash":
                                        naavProfile.Backlash = int.Parse(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.NormalizationProfile:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        normalizationProfile.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        normalizationProfile.Description = currentText.Trim();
                                        break;
                                    case "Stance":
                                        normalizationProfile.Stance = NormalizationProfile.NormalizationStanceFromString(currentText);
                                        break;
                                    case "QuantizationStepCount":
                                        normalizationProfile.QuantizationStepCount = int.Parse(currentText);
                                        break;
                                    case "Threshold":
                                        normalizationProfile.Threshold = float.Parse(currentText);
                                        break;
                                    case "ThresholdRepetitions":
                                        normalizationProfile.ThresholdRepetitions = int.Parse(currentText);
                                        break;
                                    case "SearchWidth":
                                        normalizationProfile.SearchWidth = int.Parse(currentText);
                                        break;
                                    case "Backlash":
                                        normalizationProfile.Backlash = int.Parse(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.TransformationProfileCollection:
                                switch (currentElementName)
                                {
                                    case "Name":
                                        profileCollection.Name = currentText.Trim();
                                        break;
                                    case "Description":
                                        profileCollection.Description = currentText.Trim();
                                        break;
                                    case "SubProfileName":
                                        string spName = currentText.Trim();
                                        //if (TransformationProfiles.ContainsTransformationProfile(spName))
                                        //{
                                        //    profileCollection.AddTransformationProfile(TransformationProfiles.GetTransformationProfile(spName));
                                        //}
                                        //else
                                        //{
                                        //    string errorMessage = "The transformation profile collection '" +
                                        //        profileCollection.Name + "' contains a reference to a sub-profile named '" +
                                        //        spName + ", but this profile was not found in the definition file.";
                                        //    if (TransformationProfiles.ContainsTransformationProfile(spName, false))
                                        //    {
                                        //        errorMessage += " the collection does contain a profile named '" +
                                        //            TransformationProfiles.GetCaseInsensitiveTransformationProfileName(spName) +
                                        //            "'. Profile names are case-sensitive. If this is the profile you intended " +
                                        //            "to reference, please correct the capitalization of the profile names to match.";
                                        //    }
                                        //    errorMessage += " Please correct or remove the reference.";
                                        //    throw new ArgumentException(errorMessage);
                                        //}
                                        TransformationProfileCollection.AddTransformationProfileIfItExists(
                                            ref _TransformationProfiles, ref profileCollection, spName,
                                            "The global transformation profile list");
                                        break;
                                }
                                break;
                        }
                        break;
                }
            }
        }
    }
}
