// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci.TransformationProfiles
{
    public class ClippingProfile : TransformationProfile
    {
        protected bool _ClipInputDataLower;
        protected bool _ClipInputDataUpper;
        protected float _ClipInputDataLowerValue;
        protected float _ClipInputDataUpperValue;

        #region Properties

        /// <summary>
        /// If this value is true, then the lower bound of the input data will be clipped to 
        /// the value specified by the ClipInputDataLowerValue property.
        /// If this value is false, no clipping of the lower bound will be performed.
        /// </summary>
        public bool ClipInputDataLower
        {
            get
            {
                return _ClipInputDataLower;
            }
            set
            {
                _ClipInputDataLower = value;
            }
        }

        /// <summary>
        /// If this value is true, then the upper bound of the input data will be clipped to 
        /// the value specified by the ClipInputDataUpperValue property.
        /// If this value is false, no clipping of the upper bound will be performed.
        /// </summary>
        public bool ClipInputDataUpper
        {
            get
            {
                return _ClipInputDataUpper;
            }
            set
            {
                _ClipInputDataUpper = value;
            }
        }

        /// <summary>
        /// If the ClipInputDataLower option is enabled, the lower bound of the input data will be clipped to this value.
        /// </summary>
        public float ClipInputDataLowerValue
        {
            get
            {
                return _ClipInputDataLowerValue;
            }
            set
            {
                //if (value >= ClipInputDataUpperValue)
                //{
                //    throw new ArgumentOutOfRangeException("The lower bound for clipping must be less than the upper bound.");
                //}
                _ClipInputDataLowerValue = value;
            }
        }

        /// <summary>
        /// If the ClipInputDataUpper option is enabled, the upper bound of the input data will be clipped to this value.
        /// </summary>
        public float ClipInputDataUpperValue
        {
            get
            {
                return _ClipInputDataUpperValue;
            }
            set
            {
                //if (value <= ClipInputDataLowerValue)
                //{
                //    throw new ArgumentOutOfRangeException("The upper bound for clipping must be greater than the lower bound.");
                //}
                _ClipInputDataUpperValue = value;
            }
        }

        #endregion

        public ClippingProfile()
            : base()
        {
            ClipInputDataLower = false;
            ClipInputDataUpper = false;
            ClipInputDataUpperValue = 1.0f;
            ClipInputDataLowerValue = 0.0f;
        }

        public override string WrapScriptObjectInTransformationCode(string scriptObject)
        {
            if ((!ClipInputDataLower) && (!ClipInputDataUpper))
            {
                return scriptObject;
            }
            StringBuilder builder = new StringBuilder();
            builder.Append("clipdata(");
            builder.Append(scriptObject);
            if (ClipInputDataLower)
            {
                builder.Append(", min = ");
                builder.Append(ClipInputDataLowerValue.ToString());
            }
            if (ClipInputDataUpper)
            {
                builder.Append(", max = ");
                builder.Append(ClipInputDataUpperValue.ToString());
            }
            builder.Append(")");
            return builder.ToString();
        }

        public override string GetProfileTypeName()
        {
            return "Clipping";
        }

    }
}
