// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci.TransformationProfiles
{
    public class DataRepositionAndScaleProfile : TransformationProfile
    {
        protected bool _SetOutputLowerBound;
        protected bool _SetOutputUpperBound;
        protected float _OutputLowerBound;
        protected float _OutputUpperBound;

        #region Properties

        public bool SetOutputLowerBound
        {
            get
            {
                return _SetOutputLowerBound;
            }
            set
            {
                _SetOutputLowerBound = value;
            }
        }

        public bool SetOutputUpperBound
        {
            get
            {
                return _SetOutputUpperBound;
            }
            set
            {
                _SetOutputUpperBound = value;
            }
        }

        /// <summary>
        /// The number to which negative values should be scaled so that the lowest negative value
        /// is equal to it.
        /// </summary>
        public float OutputLowerBound
        {
            get
            {
                return _OutputLowerBound;
            }
            set
            {
                //if (value >= OutputUpperBound)
                //{
                //    throw new ArgumentOutOfRangeException("The lower bound cannot be greater than or equal to the upper bound.");
                //}
                _OutputLowerBound = value;
            }
        }

        /// <summary>
        /// The number to which positive values should be scaled so that the highest positive value
        /// is equal to it.
        /// </summary>
        public float OutputUpperBound
        {
            get
            {
                return _OutputUpperBound;
            }
            set
            {
                //if (value <= OutputLowerBound)
                //{
                //    throw new ArgumentOutOfRangeException("The upper bound cannot be less than or equal to the lower bound.");
                //}
                _OutputUpperBound = value;
            }
        }

        #endregion

        public DataRepositionAndScaleProfile()
        {
            SetOutputLowerBound = false;
            SetOutputUpperBound = false;
            OutputUpperBound = 1.0f;
            OutputLowerBound = 0.0f;
        }

        public override string WrapScriptObjectInTransformationCode(string scriptObject)
        {
            if ((!SetOutputLowerBound) && (!SetOutputUpperBound))
            {
                return scriptObject;
            }
            StringBuilder builder = new StringBuilder();
            builder.Append("clipdata(");
            builder.Append(scriptObject);
            if (SetOutputLowerBound)
            {
                builder.Append(", lbound = ");
                builder.Append(OutputLowerBound.ToString());
            }
            if (SetOutputUpperBound)
            {
                builder.Append(", ubound = ");
                builder.Append(OutputUpperBound.ToString());
            }
            builder.Append(")");
            return builder.ToString();
        }

        public override string GetProfileTypeName()
        {
            return "Data Reposition And Scale";
        }
           
    }
}
