// BenLincoln.Utility - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.Utility.

// BenLincoln.Utility is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.Utility is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.Utility (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.Utility
{
    public class KeyedMessageList : IComparable
    {
        protected string _KeyName;
        protected string[] _Messages;

        #region Properties

        public string KeyName
        {
            get
            {
                return _KeyName;
            }
            set
            {
                _KeyName = value;
            }
        }

        public string[] Messages
        {
            get
            {
                return _Messages;
            }
            set
            {
                _Messages = value;
            }
        }

        #endregion

        public KeyedMessageList()
        {
            KeyName = "";
            Messages = new string[0];
        }

        public void AddMessage(string message)
        {
            if ((_Messages == null) || (_Messages.Length == 0))
            {
                _Messages = new string[1];
            }
            else
            {
                Array.Resize(ref _Messages, _Messages.Length + 1);
            }
            _Messages[_Messages.Length - 1] = message;
        }

        public int CompareTo(object obj)
        {
            if (obj is KeyedMessageList)
            {
                KeyedMessageList b2 = (KeyedMessageList)obj;
                return KeyName.CompareTo(b2.KeyName);
            }
            else
            {
                return KeyName.CompareTo(obj.ToString());
            }
        }
    }

    public class MessageCollection : CollectionBase
    {
        public MessageCollection()
            : base()
        {
        }

        public KeyedMessageList this[int index]
        {
            get
            {
                return (KeyedMessageList)this.List[index];
            }
            set
            {
                this.List[index] = value;
            }
        }

        public KeyedMessageList this[string keyName]
        {
            get
            {
                return (KeyedMessageList)this.List[IndexOf(keyName)];
            }
            set
            {
                this.List[IndexOf(keyName)] = value;
            }
        }

        public int IndexOf(KeyedMessageList item)
        {
            return base.List.IndexOf(item);
        }

        public int IndexOf(string keyName)
        {
            for (int i = 0; i < this.Count; i++)
            {
                if (this[i].KeyName == keyName)
                {
                    return i;
                }
            }
            return -1;
        }

        public int Add(KeyedMessageList item)
        {
            return this.List.Add(item);
        }

        public void Remove(KeyedMessageList item)
        {
            this.List.Remove(item);
        }

        public void CopyTo(Array array, int index)
        {
            this.List.CopyTo(array, index);
        }

        public bool Contains(KeyedMessageList item)
        {
            return this.List.Contains(item);
        }

        public bool Contains(string itemName)
        {
            if (IndexOf(itemName) > -1)
            {
                return true;
            }
            return false;
        }

        public void Insert(int index, KeyedMessageList item)
        {
            this.List.Insert(index, item);
        }

        public void Sort()
        {
            InnerList.Sort();
        }

        public void AddNewMessage(string keyName, string message)
        {
            if (Contains(keyName))
            {
                this[keyName].AddMessage(message);
            }
            else
            {
                KeyedMessageList newList = new KeyedMessageList();
                newList.KeyName = keyName;
                newList.AddMessage(message);
                this.Add(newList);
            }
        }
    }
}
