// BenLincoln.Utility - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.Utility.

// BenLincoln.Utility is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.Utility is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.Utility (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Text.RegularExpressions;

namespace BenLincoln.Utility
{
    public class ProcessLocator
    {

        public static Process[] GetProcessTree(int parentProcessID)
        {
            ArrayList processList = new ArrayList();
            try
            {
                Process parent = Process.GetProcessById(parentProcessID);
                processList.Add(parent);
                ArrayList fallback = processList;
                try
                {
                    Process[] childProcesses = GetProcessTree(parentProcessID);
                    foreach (Process child in childProcesses)
                    {
                        ArrayList fallback2 = processList;
                        try
                        {
                            processList.Add(child);
                        }
                        catch (Exception ex)
                        {
                            processList = fallback2;
                        }
                    }
                }
                catch (Exception ex)
                {
                    processList = fallback;
                }
            }
            catch (Exception ex)
            {
                throw new Exception("Unable to build the process tree. The error description was '" + ex.Message + "'.", ex);
            }

            return (Process[])processList.ToArray(Process.GetCurrentProcess().GetType());
        }

        public static Process[] GetProcesses(int parentProcessID)
        {
            return GetProcesses(parentProcessID, "");
        }

        public static Process[] GetProcesses(int parentProcessID, string instanceNameRegex)
        {
            return GetProcesses(parentProcessID, instanceNameRegex, true);
        }

        public static Process[] GetProcesses(string instanceNameRegex, bool ignoreCase)
        {
            return GetProcesses(-1, instanceNameRegex, ignoreCase);
        }

        public static Process[] GetProcesses(int parentProcessID, string instanceNameRegex, bool ignoreCase)
        {
            ArrayList idList = new ArrayList();
            PerformanceCounterCategory pcc = new PerformanceCounterCategory("Process");
            string[] instanceNames = pcc.GetInstanceNames();

            Regex rex = new Regex(instanceNameRegex, RegexOptions.IgnoreCase);
            if (!ignoreCase)
            {
                rex = new Regex(instanceNameRegex);
            }

            foreach (string iName in instanceNames)
            {
                if ((instanceNameRegex == "") || (rex.IsMatch(iName)))
                {
                    int pProcessID = 0;
                    int processID = 0;
                    foreach (PerformanceCounter counter in pcc.GetCounters(iName))
                    {
                        string counterName = counter.CounterName.ToLower();
                        if (counterName == "creating process id")
                        {
                            pProcessID = (int)counter.RawValue;
                        }
                        if (counterName == "id process")
                        {
                            processID = (int)counter.RawValue;
                        }
                        //Console.WriteLine("Debug: Counter Name '" + counter.CounterName + "' + Instance Name '" +
                        //    counter.InstanceName + "' + Category Name '" + counter.CategoryName + "' + Raw Value " +
                        //    counter.RawValue.ToString());
                    }
                    if ((parentProcessID == -1) || (pProcessID == parentProcessID))
                    {
                        idList.Add(processID);
                    }
                }
            }

            int[] IDs = (int[])idList.ToArray(1.GetType());

            if (IDs.Length == 0)
            {
                return new Process[0];
            }
            if (IDs.Length == 1)
            {
                return new Process[] { Process.GetProcessById(IDs[0]) };
            }         

            ArrayList pList = new ArrayList();

            foreach (int pid in IDs)
            {
                Process p = Process.GetProcessById(pid);
                pList.Add(p);
            }

            return (Process[])pList.ToArray(Process.GetCurrentProcess().GetType());
        }
    }
}
