// The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of The Mirror's Surface Breaks.

// The Mirror's Surface Breaks is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// The Mirror's Surface Breaks is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with The Mirror's Surface Breaks (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace TMSBGUI
{
    public partial class frmAbout : Form
    {
        public frmAbout()
        {
            InitializeComponent();
            SetupLogoImage();
            SetupText();
        }

        protected void SetupLogoImage()
        {
            try
            {
                Assembly currentAssembly = Assembly.GetExecutingAssembly();
                Stream iStream = currentAssembly.GetManifestResourceStream("TMSBGUI.TMSB_Logo.PNG");
                this.pbLogoImage.Image = Image.FromStream(iStream);
                iStream.Close();
            }
            catch (Exception ex)
            {
                this.pbLogoImage.Image = null;
                MessageBox.Show("Unable to load embedded image.", "IO Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        protected void SetupText()
        {
            try
            {
                Assembly currentAssembly = Assembly.GetExecutingAssembly();
                Stream iStream = currentAssembly.GetManifestResourceStream("TMSBGUI.AboutText.txt");
                StreamReader iReader = new StreamReader(iStream);
                txtAbout.Text = iReader.ReadToEnd();
                iReader.Close();
                iStream.Close();
            }
            catch (Exception ex)
            {
                txtAbout.Text = "Unable to load embedded text definition.";
            }

            string currentText = txtAbout.Text;

            try
            {
                Assembly currentAssembly = Assembly.GetExecutingAssembly();
                AssemblyName assemblyName = currentAssembly.GetName();

                int verMaj = assemblyName.Version.Major;
                int verMin = assemblyName.Version.Minor;
                int verBuild = assemblyName.Version.Build;

                StringBuilder builder = new StringBuilder(currentText);
                builder.Append(Environment.NewLine);
                builder.Append("Software version: ");
                builder.Append(verMaj.ToString());
                builder.Append(".");
                builder.Append(verMin.ToString());
                builder.Append(".");
                builder.Append(verBuild.ToString());
                txtAbout.Text = builder.ToString();
            }
            catch (Exception ex)
            {
                txtAbout.Text = currentText;
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

    }
}